#! /usr/bin/rscript

library(reshape)
library(ggplot2)

# Data set sorted by experimental condition (stock, libxul, static)
# and then in increasing order of hard drive performance.  Discard the
# first data point in each set, which is nonrepresentative because it
# does some setup work that is cached for later runs, as well as
# priming operating-system and CPU caches.

timings.raw <- data.frame(stock.4200=scan('data-10.6.1-4200rpm-USB/Stock.csv', skip=1),
                          stock.5400=scan('data-10.6.2-5400rpm/Stock.csv', skip=1),
                          stock.7200=scan('data-10.6.1-7200rpm/Stock.csv', skip=1),
                          stock.ssd=scan('data-10.6.2-ssd/Stock.csv', skip=1),
                          libxul.4200=scan('data-10.6.1-4200rpm-USB/LIBXUL.csv', skip=1),
                          libxul.5400=scan('data-10.6.2-5400rpm/LIBXUL.csv', skip=1),
                          libxul.7200=scan('data-10.6.1-7200rpm/LIBXUL.csv', skip=1),
                          libxul.ssd=scan('data-10.6.2-ssd/LIBXUL.csv', skip=1),
                          static.4200=scan('data-10.6.1-4200rpm-USB/Static.csv', skip=1),
                          static.5400=scan('data-10.6.2-5400rpm/Static.csv', skip=1),
                          static.7200=scan('data-10.6.1-7200rpm/Static.csv', skip=1),
                          static.ssd=scan('data-10.6.2-ssd/Static.csv', skip=1)
                          )

timings.flat <- melt(timings.raw, id.vars=c(), variable_name='condition')

# compute mean and standard deviation in each condition
timings.msd <- cast(timings.flat, condition ~ .,
                    fun.aggregate=function(v) list(mean=mean(v), sd=sd(v)))

timings.norm <- data.frame(condition = timings.flat$condition,
                           value = numeric(length(timings.flat$value)))

for (c in timings.msd$condition) {
  M <- timings.msd[timings.msd$condition==c, 'mean']
  S <- timings.msd[timings.msd$condition==c, 'sd']
  ROWS <- timings.norm$condition == c
  timings.norm[ROWS, 'value'] <- (timings.flat[ROWS, 'value'] - M) / S
}

# overlay frame with the normal distribution
overlay.norm <- data.frame(x=seq(-3.5,3.5,0.1), y=dnorm(seq(-3.5,3.5,0.1)))

# diagnostic plot of the pdf of each condition
p <- ggplot(timings.norm, aes(x=value)) + facet_wrap(~ condition, ncol=4) +
  geom_density(fill='gray', colour=NA) + geom_rug() +
  layer(data=overlay.norm, mapping=aes(x=x, y=y), geom='line',
        geom_params=list(colour='red', size=0.5)) +
  scale_x_continuous(name='Startup time, normalized per condition',
                     limits=c(-9, 9), breaks=seq(-9, 9, 3)) +
  ylab('Observation density')

pdf(file='diagnosis-u.pdf', width=9, height=6.5)
print(p)
dev.off()
system('qpdf diagnosis-u.pdf diagnosis.pdf')
